/* pal
 *
 * Copyright (C) 2003, Scott Kuhl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#include <time.h>

#include "main.h"
#include "event.h"
#include "html.h"


/* finishes with date on the sunday of the next week */
void pal_html_week(GDate* date, gboolean force_month_label,
			  const GDate* today)
{
    gint i=0;

    if(settings->week_start_monday)
	/* go to last day in week (sun) */
	while(g_date_get_weekday(date) != 7)
	    g_date_add_days(date,1);
    else
	/* go to last day in week (sat) */
	while(g_date_get_weekday(date) != 6)
	    g_date_add_days(date,1);


    for(i=0; i<7; i++)
    {
	if(g_date_get_day(date) == 1)
	    force_month_label = TRUE;

	g_date_subtract_days(date,1);
    }

    g_date_add_days(date,1);
    /* date is now at beginning of week */

    g_print("<tr>\n");


    if(force_month_label)
    {
	gchar buf[1024];

	g_date_add_days(date,6);
	g_date_strftime(buf, 1024, "%b<br>%Y", date);
	g_date_subtract_days(date,6);

	g_printf("<td bgcolor=#cccccc align=center><b>%s</b></td>\n", buf);
    }
    else
    {
	g_printf("<td></td>\n");

    }


    for(i=0; i<7; i++)
    {
	GList* events = get_events(date);
	gint num_events = g_list_length(events);
	gint on_event_file = 0;
	gint num_events_printed = 0;

	/* make today bright */
	if(g_date_compare(date,today) == 0)
	    g_print("<td bgcolor=#dddddd valign=top><b>%02d</b><br />\n", g_date_get_day(date));
	else
	    g_print("<td valign=top><b>%02d</b><br />\n", g_date_get_day(date));

	/* while theres more events to be displayed */
	while(num_events > num_events_printed)
	{
	    GList* item = g_list_first(events);
	    int i;

	    /* iterate through list */
	    for(i=0; i<num_events; i++)
	    {
		/* if this event in list matches the number we're on, print it */
		if(((PalEvent*) (item->data))->file_num == on_event_file)
		{
		    g_print("<b>*</b> ");
		    g_print(((PalEvent*) (item->data))->text);
		    g_print("<br />\n");
		    num_events_printed++;
		}

		item = g_list_next(item);
	    }

	    /* done with this number, on to the next */
	    on_event_file++;
	}

	g_date_add_days(date,1);
	g_list_free(events);
    }

    g_print("</tr>\n");

}




void pal_html_out()
{
    gint on_week = 0;
    GDate* today = g_date_new();
    GDate* date = g_date_new();

    gchar start[64] = "<td bgcolor=#cccccc align=center><b>";
    gchar end[64] = "</td></b>";

    g_date_set_time(today, time(NULL));
    g_date_set_time(date, time(NULL));

    g_print("%s", "<table border=1 width=100% cellspacing=0 cellpadding=1>\n");
    g_print("<tr><td></td>\n");

    if(!settings->week_start_monday)
	g_print("%s%s%s\n", start, _("Sunday"), end);

    g_print("%s%s%s\n", start, _("Monday"), end);
    g_print("%s%s%s\n", start, _("Tuesday"), end);
    g_print("%s%s%s\n", start, _("Wednesday"), end);
    g_print("%s%s%s\n", start, _("Thursday"), end);
    g_print("%s%s%s\n", start, _("Friday"), end);
    g_print("%s%s%s\n", start, _("Saturday"), end);

    if(settings->week_start_monday)
	g_print("%s%s%s\n", start, _("Sunday"), end);

    g_print("</tr>\n");

    while(on_week < settings->cal_lines)
    {
	pal_html_week(date, TRUE, today);
	on_week++;
    }

    g_print("</table>\n");
    g_print("<p align='right'><font size='-1'><i>Calendar created with <a href='http://scott.kuhlweb.com/compu/pal/index.html'>pal</a>.</i></font></p>\n");

}
