/* pal
 *
 * Copyright (C) 2003, Scott Kuhl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdarg.h>
#include <time.h>

#include <string.h>

#include "main.h"
#include "colorize.h"
#include "event.h"

/* set attribute w/o changing color */
void pal_output_attr(gint attr, gchar *formatString, ...)
{
    gchar color[20];
    va_list argptr;

    va_start( argptr, formatString );
    colorize_attr(attr, color);
    g_print(color);

    /* glib 2.2 provides g_vfprintf */
    vfprintf(stdout, formatString, argptr);

    colorize_reset(color);
    g_print(color);
    va_end(argptr);
}

/* set foreground color and attribute */
void pal_output_fg( gint attr, gint color, gchar *formatString, ...)
{
    gchar str[20];
    va_list argptr;

    va_start( argptr, formatString );
    colorize_fg(attr, color, str);
    g_print(str);

    /* glib 2.2 provides g_vfprintf */
    vfprintf(stdout, formatString, argptr);

    colorize_reset(str);
    g_print(str);
    va_end(argptr);
}

void pal_output_error(char *formatString, ... )
{
    gchar color[20];
    va_list argptr;

    va_start( argptr, formatString );
    colorize_fg(BRIGHT, RED, color);
    g_print(color);

    /* glib 2.2 provides g_vfprintf */
    vfprintf(stderr, formatString, argptr);

    colorize_reset(color);
    g_print(color);
    va_end(argptr);
}


/* finishes with date on the sunday of the next week */
void pal_output_text_week(GDate* date, gboolean force_month_label,
			  const GDate* today, gchar* buffer)
{
    gchar* loc = NULL;
    gint i=0;
    gchar start_color[20];
    gchar end_color[20];

    if(settings->week_start_monday)
	/* go to last day in week (sun) */
	while(g_date_get_weekday(date) != 7)
	    g_date_add_days(date,1);
    else
	/* go to last day in week (sat) */
	while(g_date_get_weekday(date) != 6)
	    g_date_add_days(date,1);


    for(i=0; i<7; i++)
    {
	if(g_date_get_day(date) == 1)
	    force_month_label = TRUE;

	g_date_subtract_days(date,1);
    }

    g_date_add_days(date,1);
    /* date is now at beginning of week */

    if(force_month_label)
    {
	gchar buf[1024];

	colorize_fg(BRIGHT, GREEN, start_color);
	colorize_reset(end_color);

	g_date_add_days(date,6);
	g_date_strftime(buf, 5, "%b ", date);
	g_date_subtract_days(date,6);

	loc = buffer + g_sprintf(buffer, "%s%s%s", start_color, buf, end_color);
    }
    else
    {
	g_sprintf(buffer, "    ");
	loc = buffer + 4;
    }


    for(i=0; i<7; i++)
    {
	GList* events = NULL;
	gchar start=' ', end=' ';
	gint color = settings->event_color;
	events = get_events(date);

	if(g_date_compare(date,today) == 0)
	    start = '@', end = '@';

	else if(events != NULL)
	{
	    GList* item  = g_list_first(events);
	    gboolean same_char = TRUE;
	    gboolean same_color = TRUE;

	    /* skip to a event that isn't hidden or to the end of the list */
	    while(g_list_length(item) > 1 && ((PalEvent*) item->data)->hide)
		item = g_list_next(item);

	    /* save the markers for the event */
	    if(((PalEvent*) item->data)->hide)
		start = ' ', end = ' ';
	    else
	    {
		start = ((PalEvent*) item->data)->start;
		end   = ((PalEvent*) item->data)->end;
		color = ((PalEvent*) item->data)->color;
	    }

	    /* if multiple events left */
	    while(g_list_length(item) > 1)
	    {
		/* find next non-hidden event */
		while(g_list_length(item) > 1 && ((PalEvent*) item->data)->hide)
		    item = g_list_next(item);

		/* if this event is hidden, there aren't any more non-hidden events left */
		/* if this event isn't hidden, then determine if it has different markers */
		if(!((PalEvent*) item->data)->hide)
		{
		    gchar new_start = ((PalEvent*) item->data)->start;
		    gchar new_end   = ((PalEvent*) item->data)->end;
		    gint new_color  = ((PalEvent*) item->data)->color;

		    if(new_start != start || new_end != end)
			same_char = FALSE;
		    if(new_color != color)
			same_color = FALSE;

		    /* jump to next event to force the execution of the while loop above
		       otherwise we get stuck in an infinite loop on a non-hidden event */
		    item = g_list_next(item);
		}
		if(same_char == FALSE)
		    start = '*', end = '*';
		if(same_color == FALSE)
		    color = -1;
	    }
	}

	/* check if color codes are needed */
	if(start != ' ' && end != ' ')
	{
	    if(color == -1)
		colorize_fg(BRIGHT, settings->event_color, start_color);
	    else
		colorize_fg(BRIGHT, color, start_color);

	    colorize_reset(end_color);

	    /* make today bright */
	    if(g_date_compare(date,today) == 0)
	    {
		gchar start_color_today[20];
		colorize_attr(BRIGHT, start_color_today);

		loc += g_snprintf(loc, 100, "%s%c%s%s%02d%s%s%c%s ", start_color, start, end_color,
			  start_color_today, g_date_get_day(date), end_color, start_color, end, end_color);
	    }
	    else
		loc += g_snprintf(loc, 100, "%s%c%s%02d%s%c%s ", start_color, start, end_color,
				  g_date_get_day(date), start_color, end, end_color);
	}
	else
	    loc += g_snprintf(loc, 100, "%c%02d%c ", start, g_date_get_day(date), end);

	/* don't need an extra space at end of text */
	if(i==6)
	{
	   loc--;
	   *loc = '\0';
	}

	g_date_add_days(date,1);
	g_list_free(events);
    }

}



void pal_output_week(GDate* date, gboolean force_month_label, const GDate* today)
{
    gchar week[1024];
    gchar week2[1024];

    pal_output_text_week(date, force_month_label, today, week);

    if(!settings->no_columns)
    {
        /* skip ahead to next column */
	g_date_subtract_days(date, 6);
	g_date_add_days(date, settings->cal_lines*7);
	pal_output_text_week(date, force_month_label, today, week2);

	/* skip back to where we were */
	g_date_subtract_days(date, settings->cal_lines*7);

	g_print("%s", week);
	pal_output_fg(DIM,YELLOW,"%s","|");
	g_print("%s\n", week2);

    }
    else
	g_print("%s\n", week);

}



void pal_output_cal(GDate* date, gint num_lines, const GDate* today)
{
    gint on_week = 0;
    gchar* week_hdr = NULL;

    if(num_lines <= 0)
	return;

    if(settings->week_start_monday)
	week_hdr = g_strdup(_("     Mo   Tu   We   Th   Fr   Sa   Su"));
    else
	week_hdr = g_strdup(_("     Su   Mo   Tu   We   Th   Fr   Sa"));


    /* if showing enough lines, show previous week.
       perhaps a more elegant check in the future */
    if(num_lines > 3)
	g_date_subtract_days(date, 7);

    if(settings->no_columns)
	pal_output_fg(BRIGHT,GREEN, "%s\n", week_hdr);

    else
    {
	pal_output_fg(BRIGHT,GREEN,"%s ", week_hdr);
	pal_output_fg(DIM,YELLOW,"%s","|");
	pal_output_fg(BRIGHT,GREEN,"%s\n", week_hdr);
    }

    g_free(week_hdr);

    while(on_week < num_lines)
    {
	if(on_week == 0)
	    pal_output_week(date, TRUE, today);
	else
	    pal_output_week(date, FALSE, today);

	on_week++;

    }
}

/* replaces tabs with spaces */
void pal_output_strip_tabs(gchar* string)
{
    gchar *ptr = string;
    while(*ptr != '\0')
    {
	if(*ptr == '\t')
	    *ptr = ' ';
	ptr++;
    }
}


/* This function does not yet handle tabs and color codes.  Tabs should be stripped
 * from 'string' before this is called */
gchar* pal_output_wrap(gchar* string, gint chars_used)
{
    gchar* buffer = g_malloc(sizeof(gchar) * strlen(string) * 2);
    gchar* ptr = buffer; /* beginning of actual line */
    gchar* begin_line_text = buffer; /* beginning of line's text */
    g_sprintf(buffer, "%s", string);

    /* Don't bother with wrapping if terminal is very narrow. As
     * things are working now, not doing this could cause a
     * segmentation faults on very narrow terminals. */
    if(settings->term_cols < 16)
	return buffer;

    while(strlen(ptr)+chars_used > settings->term_cols)
    {
	gchar saved_char = ' ';

	/* jump to where newline needs to be */
	ptr += (settings->term_cols-chars_used);

	/* find a ' ' to break at. */
	while(ptr != begin_line_text && *ptr != ' ')
	    ptr--;

	/* if we found no ' '; extremely long word */
	/* break in the middle of the word---sorry! */
	if(ptr == begin_line_text && *ptr != ' ')
	    ptr += (settings->term_cols-chars_used);

	/* no need for this anymore */
	chars_used = 0;

	/* we might be on a character if we hit a long word. */
	/* save this letter for later */
	if(*ptr != ' ')
	    saved_char = *ptr;
	else
	    saved_char = ' ';

	*ptr = '\n';

	/* find end of string */
	while(*ptr != '\0')
	    ptr++;
	/* bump all letters back 2 places */
	while(*ptr != '\n')
	{
	    if(saved_char == ' ')
		*(ptr+2) = *ptr;
	    else /* we need to shift 3 in order to print the saved letter */
		*(ptr+3) = *ptr;
	    ptr--;
	}

	/* insert hanging indent spaces */
	ptr++;
	*ptr = ' ';
	ptr++;
	*ptr = ' ';

	/* finally, print that pesky saved character */
	if(saved_char != ' ')
	{
	    ptr++;
	    *ptr = saved_char;
	    ptr--;
	}

	ptr--; /* leave ptr at real beginning of line
		  (immediately after \n) */

	begin_line_text = ptr+2; /* beginning of actual text */
    }

    return buffer;
}


void pal_output_event(const PalEvent* event, const GDate* date)
{
    gchar* event_text;
    gchar date_text[128];

    date_text[0] = '\0';

    if(event->color == -1)
	pal_output_fg(BRIGHT, settings->event_color, "%s ", "*");
    else
	pal_output_fg(BRIGHT, event->color, "%s ", "*");

    pal_output_strip_tabs(event->text);
    pal_output_strip_tabs(event->type);

    if(settings->compact_list)
    {
	if(settings->hide_event_type)
	{
	    gchar* s;

	    g_date_strftime(date_text, 128,
			    settings->compact_date_fmt, date);
	    s = g_strconcat(event->text, NULL);
	    event_text = pal_output_wrap(s, 2+strlen(date_text)+1);
	    g_free(s);
	}
	else
	{
	    gchar* s;

	    g_date_strftime(date_text, 128,
			    settings->compact_date_fmt, date);
	    s = g_strconcat(event->type, ": ", event->text, NULL);
	    event_text = pal_output_wrap(s, 2+strlen(date_text)+1);
	    g_free(s);
	}

    }
    else
    {
	if(settings->hide_event_type)
	    event_text = pal_output_wrap(event->text, 2);
	else
	{
	    gchar* s = g_strconcat(event->type, ": ", event->text, NULL);
	    event_text = pal_output_wrap(s, 2);
	    g_free(s);
	}
    }

    if(date_text[0] != '\0')
	pal_output_attr(BRIGHT, "%s ", date_text);

    g_print("%s\n", event_text);

    g_free(event_text);
}

void pal_output_date_line(const GDate* date)
{
    gchar pretty_date[128];
    gint diff = 0;
    GDate* today = g_date_new();
    g_date_set_time(today, time(NULL));

    g_date_strftime(pretty_date, 128, settings->date_fmt, date);

    pal_output_attr(BRIGHT, "%s", pretty_date);
    g_print("%s", " - ");

    diff = g_date_days_between(today, date);
    if(diff == 0)
	pal_output_fg(BRIGHT, RED, "%s", _("Today"));
    else if(diff == 1)
	pal_output_fg(BRIGHT, YELLOW, "%s", _("Tomorrow"));
    else if(diff == -1)
	g_print("%s", _("Yesterday"));
    else if(diff > 1)
	g_print("%d %s", diff, _("days away"));
    else if(diff < -1)
	g_print("%d %s", -1*diff, _("days ago"));

    g_print("%s", "\n");

    g_date_free(today);
}


/* outputs the events in the order of PalEvent->file_num */
void pal_output_date(GDate* date, gboolean show_empty_days)
{
    GList* events = get_events(date);
    gint num_events = g_list_length(events);
    gint on_event_file = 0;
    gint num_events_printed = 0;

    if(events != NULL || show_empty_days)
    {
	if(!settings->compact_list)
	    pal_output_date_line(date);

	 /* while theres more events to be displayed */
	while(num_events > num_events_printed)
	{
	    GList* item = g_list_first(events);
	    int i;

	    /* iterate through list */
	    for(i=0; i<num_events; i++)
	    {
		/* if this event in list matches the number we're on, print it */
		if(((PalEvent*) (item->data))->file_num == on_event_file)
		{
		    pal_output_event((PalEvent*) (item->data), date);
		    num_events_printed++;
		}

		item = g_list_next(item);
	    }

	    /* done with this number, on to the next */
	    on_event_file++;
	}

	if(num_events == 0)
	{
	    if(settings->compact_list)
	    {
		gchar pretty_date[128];
		gchar* s1;
		gchar* s2;

		g_date_strftime(pretty_date, 128,
				settings->compact_date_fmt, date);
		s1 = g_strconcat(pretty_date, " No events.\n" , NULL);
		s2 = pal_output_wrap(s1, 2);
		g_print(s2);
		g_free(s1);
		g_free(s2);
	    }
	    else
		g_print("%s\n", _("No events."));
	}

	if(!settings->compact_list)
	    g_print("%s", "\n");

    }
}
